<?php
namespace Teknisa\FAI\Modules\Apuracao\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Teknisa\Geral\Util\Environment;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Fai70000CalculoApuracao extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName;
    protected $crudService;
    protected $parameterBag;
    protected $entityManager;
    protected $environment;
    protected $dataSourceManager;
	protected $parameterBagColumns;
	protected $novoCodigo;
    
    public function __construct($crudService, ParameterBag $parameterBag, $environment, $entityManager, $dataSourceManager, $novoCodigo) {
        parent::__construct($crudService, $parameterBag);
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
		$this->novoCodigo = $novoCodigo;
    }

	public function verificaTabelaCorrecao(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->addCondition("CDFILIAL", "=", $this->environment->getCdFilial());
		$this->dataSourceName = "fai70000_tab_correcao";
		$this->parameterBagColumns = array ("CDFILIAL","NRORG");
		parent::find($request, $response);
	}

	public function verificaLocal(DTO\Request\Filter $request, DTO\Response $response) {
		$filterCriteria = $request->getFilterCriteria();
		$this->dataSourceName = "fai70000_utiliza_local";
		$this->parameterBagColumns = array ("CDFILIAL");
		parent::find($request, $response);
	}

	private function getNrApuracFat($rows) {
		$rowsToSave = array();
		foreach ($rows as $row) {
			$conditions = array(
				array(
					"columnName" => "CDCLIENTE",
					"operator" => "=",
					"value" => $row["CDCLIENTE"]
				),
				array(
					"columnName" => "CDFILIAPUR",
					"operator" => "=",
					"value" => $row["CDFILIAPUR"]
				)
			);
			$dataSet = $this->crudService->getFilteredDataSet("fai70000_ultima_apuracao_cliente", $conditions);
			$ultimaApuracao = $dataSet->getRows();
			$ultimaApuracao = count($ultimaApuracao) > 0 ? $ultimaApuracao[0] : null;
			if (!empty($ultimaApuracao) && $ultimaApuracao['IDBLOQUEADO'] != 'S') {
				$rowsToSave[] = array(
					"CDFILIAL"    => $row["CDFILIAL"],
					"DTINIAPURAC" => $row["DTINIAPURAC"],
					"DTFIMAPURAC" => $row["DTFIMAPURAC"],
					"NRAPURACFAT" => $row["NRAPURACFAT"],
					"CDCLIENTE"   => $row["CDCLIENTE"],
					"__is_new"    => false
				);
			} else {
				if (!empty($ultimaApuracao) && $ultimaApuracao["IDBLOQUEADO"] === 'S') {
					$ultimaDtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTINIAPURAC"], 0, 10));
					$ultimaDtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($ultimaApuracao["DTFIMAPURAC"], 0, 10));
					$dtiniapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTINIAPURAC"], 0, 10));
					$dtfimapur = \DateTime::createFromFormat('!d/m/Y', substr($row["DTFIMAPURAC"], 0, 10));
					if (($dtiniapur >= $ultimaDtiniapur && $dtiniapur <= $ultimaDtfimapur) ||
					($dtfimapur >= $ultimaDtiniapur && $dtfimapur <= $ultimaDtfimapur)) {
						throw new \Exception("Já existe nota emitida para o período apurado.", 1);
					} else {
						$row["CDFILIAL"] = $this->environment->getCdFilial();
						$cdcontador = "APURACFATUR".$row["CDFILIAL"];
						$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
						$rowsToSave[] = array(
							"CDFILIAL"    => $row["CDFILIAL"],
							"NRAPURACFAT" => $row["NRAPURACFAT"],
							"CDCLIENTE"   => $row["CDCLIENTE"],
							"DTINIAPURAC" => $row["DTINIAPURAC"],
							"DTFIMAPURAC" => $row["DTFIMAPURAC"],
							"CDFILIAPUR"  => $row["CDFILIAPUR"],
							"CDTIPOFATUCT"=> "001",
							"__is_new"    => true
						);
					}
				} else {
					$row["CDFILIAL"] = $this->environment->getCdFilial();
					$cdcontador = "APURACFATUR".$row["CDFILIAL"];
					$row['NRAPURACFAT'] = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
					$rowsToSave[] = array(
						"CDFILIAL"    => $row["CDFILIAL"],
						"NRAPURACFAT" => $row["NRAPURACFAT"],
						"CDCLIENTE"   => $row["CDCLIENTE"],
						"DTINIAPURAC" => $row["DTINIAPURAC"],
						"DTFIMAPURAC" => $row["DTFIMAPURAC"],
						"CDFILIAPUR"  => $row["CDFILIAPUR"],
						"CDTIPOFATUCT"=> "001",
						"__is_new"    => true
					);
				}
			}
		}
		return $rowsToSave;
	}
	
	private function saveCtrlApurFat($rowsToSave) {
		$dataSet = new DataSet(null, $rowsToSave);
		$request = new DTO\Request\DataSet($dataSet, '', '', '');
		$rows = $this->crudService->getRowsToSave($request,'ctrlapurfat');
		$this->crudService->save('ctrlapurfat', $rows);
	}

	private function verificaNotaFiscal($row) {
		$conditions = array (
			array (
				"columnName" 	=> "CDFILIAL",
				"operator" 		=> "=",
				"value" 		=> $row["CDFILIAL"]
			),
			array (
				"columnName" 	=> "NRAPURACFAT",
				"operator" 		=> "=",
				"value" 		=> $row['NRAPURACFAT']
			),
			array (
                "columnName"    =>'NRORG',
                "operator" 		=> "=",
                "value" 		=> $this->environment->getNrOrg()
          	)
		);

		$filterCriteria = new FilterCriteria('notafiscal', $conditions);
       	$filterCriteria->setDataSourceName('notafiscal');
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
		return $dataSet->getRows();
	}
	
	private function getCdCtrlNumeNF($row) {
		$conditions = array (
			array (
				"columnName" 	=> "CDFILIAL",
				"operator" 		=> "=",
				"value" 		=> $row["CDFILIAL"]
			),
			array (
         		"columnName" 	=> "NRORG",
         		"operator" 		=> "=",
         		"value" 		=> $this->environment->getNrOrg()
         	)
		);
		$filterCriteria = new FilterCriteria('fai70000_filial_serienf_ctrlnumenf', $conditions);
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
       	return $dataSet->getRows()[0]["CDCTRLNUMENF"];
	}
	
	private function verificaIntervalo($row, $cdctrlnumenf) {
       	$conditions = array (
			array (
				"columnName"    => "CDCTRLNUMENF",
				"operator" 		=> "=",
				"value" 		=> $cdctrlnumenf
			),
			array (
				"columnName" 	=> "CDLOCALEMIS",
				"operator" 		=> "=",
				"value" 		=> $row['CDLOCAL']
			),
			array (
                "columnName" 	=> "NRORG",
                "operator" 		=> "=",
                "value" 		=> $this->environment->getNrOrg()
     		)
		);
		$filterCriteria = new FilterCriteria('seqnumeloca', $conditions);
       	$dataSet = $this->dataSourceManager->findBy($filterCriteria);
       	return $dataSet;
	}
	
	private function saveLocalEmissao($row, $dataSet, $cdctrlnumenf) {
		$nrcdfrini = $dataSet->getRows()[0]["NRCDFRINI"];
        $nrcdfrfim = $dataSet->getRows()[0]["NRCDFRFIM"];
        $cdcontador = "CDFRLOCEMIS".$cdctrlnumenf.$row["CDLOCAL"];
        $nrsequencial = $this->novoCodigo->getCodeByTableName($cdcontador, $this->environment->getNrOrg());
	    $conditions = array (
			array (
				"columnName" 	=> "CDFILIAL",
				"operator" 		=> "=",
				"value" 		=> $row['CDFILIAL']
			),
			array (
				"columnName" 	=> "NRAPURACFAT",
				"operator" 		=> "=",
				"value" 		=> $row['NRAPURACFAT']
			),
			array (
                "columnName" 	=> "NRORG",
                "operator" 		=> "=",
                "value" 		=> $this->environment->getNrOrg()
			)
		);
		$filterCriteria = new FilterCriteria('fai70000_ctrlapcdrf', $conditions);
		$this->parameterBagColumns = array ("CDFILIAL", "NRAPURACFAT", "NRORG");
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		if(empty($dataSet->getRows())) {
			$conditions = array (
				array (
					"columnName" 	=> "CDCTRLNUMENF",
					"operator" 		=> "=",
					"value" 		=> $cdctrlnumenf
				),
				array (
					"columnName" 	=> "CDLOCALEMIS",
					"operator" 		=> "=",
					"value" 		=> $row["CDLOCAL"]
				),
				array (
					"columnName" 	=> "DTCTRLAPCDFR",
					"operator" 		=> "=",
					"value" 		=> $row["DTINIAPURAC"]
				),
				array (
          		  	"columnName" 	=> "NRORG",
          		  	"operator" 		=> "=",
          		  	"value" 		=> $this->environment->getNrOrg()
          		)
			);
			$filterCriteria = new FilterCriteria('ctrlapcdrf', $conditions);
			$dataSet = $this->dataSourceManager->findBy($filterCriteria);
			if (!empty($dataSet->getRows())) {
				$rowToSave = array (
					array (
						"CDFILIALCENT"	=> $this->environment->getCdFilial(),
						"NRAPURACFATU"	=> $row["NRAPURACFAT"],
						"CDCTRLNUMENF"	=> $cdctrlnumenf,
						"CDLOCALEMIS" 	=> $row["CDLOCAL"],
						"DTCTRLAPCDFR"	=> $row["DTINIAPURAC"],
						"__is_new"    	=> false
					)
				);
				$newDataSet = new DataSet(null, $rowToSave);
				$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
				$newRow = $this->crudService->getRowsToSave($newRequest,'ctrlapcdrf');
				$this->crudService->save('ctrlapcdrf', $newRow);
			} else {
				if($nrsequencial > $nrcdfrini && $nrsequencial < $nrcdfrfim) {
					$rowToSave = array (
						array (
							"CDCTRLNUMENF"	=> $cdctrlnumenf,
							"CDLOCALEMIS" 	=> $row["CDLOCAL"],
							"CDNRCDFR" 		=> str_pad($nrsequencial, 20, "0", STR_PAD_LEFT),
							"CDFILIALCENT"	=> $this->environment->getCdFilial(),
							"NRAPURACFATU"	=> $row["NRAPURACFAT"],
							"IDPERIODFATU"	=> $row["IDPERIODFATC"],
							"IDSTATUSIMPR"	=> "N",
							"IDSITUCDFR"  	=> "P",
							"DTCTRLAPCDFR"	=> $row["DTINIAPURAC"],
							"__is_new"    	=> true
						)
					);
					$newDataSet = new DataSet(null, $rowToSave);
					$newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
					$newRow = $this->crudService->getRowsToSave($newRequest,'ctrlapcdrf');
					$this->crudService->save('ctrlapcdrf', $newRow);
					return true;
				} else {
					return false;
				}
			}
		}
	}
	
	public function apurarRecServProdServ(DTO\Request $request, DTO\Response $response) {
		$rows = $request->getDataSet()->getRows();
		foreach($rows as $row) {
			if(!$row["NRAPURACFAT"]) {
				$row["NRAPURACFAT"] = $this->novoCodigo->getCodeByTableName("CTRLAPURFAT", $this->environment->getNrOrg());
				$this->insertCtrlapurfat($row);
			} else {
				$this->updateCtrlapurfat($row);
			}
			if($row["IDCLIENPRIN"] === "S" || $row["IDUTCONTCLPR"] === "S") {
				if($row['CDTIPOFATUCT'] === '004') {
					$this->procedureApuracRS($row);
				} else if($row['CDTIPOFATUCT'] === '007') {
					$this->procedureApuracPS($row);
				}
			} else {
				$this->executaApuracaoGestaoDireta($row, $response);
			}
		}
	}
	
	public function insertCtrlapurfat($row) {
		$sql =	"INSERT INTO CTRLAPURFAT
			(CDFILIAL, NRAPURACFAT, CDCLIENTE, DTINIAPURAC, DTFIMAPURAC, CDFILIAPUR, CDTIPOFATUCT)
			VALUES
			(:CDFILIAL, :NRAPURACFAT, :CDCLIENTE, :DTINIAPURAC, :DTFIMAPURAC, :CDFILIAPUR, :CDTIPOFATUCT))";
		$params = array(
			"CDFILIAL"  	=> $row["CDFILIALLOG"],
			"NRAPURACFAT"	=> $row["NRAPURACFAT"],
			"CDCLIENTE"		=> $row["CDCLIENTE"],
			"DTINIAPURAC"	=> $row["DTINIAPURAC"],
			"DTFIMAPURAC"	=> $row["DTFIMAPURAC"],
			"CDFILIAPUR"	=> $row["CDFILIAPUR"],
			"CDTIPOFATUCT"  => $row["CDTIPOFATUCT"]
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
		$query->execute($params);
	}
	
	public function updateCtrlapurfat($row) {
		$sql = "UPDATE CTRLAPURFAT
			SET
				DTINIAPURAC   = :DTINIAPURAC,
				DTFIMAPURAC   = :DTFIMAPURAC
			WHERE
				CDFILIAL    = :CDFILIAL AND
				NRAPURACFAT = :NRAPURACFAT AND
				CDCLIENTE   = :CDCLIENTE AND
				NVL(CDTIPOFATUCT, :CDTIPOFATUCT) = :CDTIPOFATUCT";
		$params = array(
			"DTINIAPURAC"	=> $row["DTINIAPURAC"],
			"DTFIMAPURAC"	=> $row["DTFIMAPURAC"],
			"CDFILIAL"  	=> $row["CDFILIAL"],
			"NRAPURACFAT"	=> $row["NRAPURACFAT"],
			"CDCLIENTE"		=> $row["CDCLIENTE"],
			"CDTIPOFATUCT"  => $row["CDTIPOFATUCT"]
		);
		$query = $this->entityManager->getConnection()->prepare($sql);
		$query->execute($params);
	}

	private function procedureApuracPS($row) {
		$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_PRODSERV');
		$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILIAL"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row["CDCLIENTE"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row["DTINIAPURAC"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row["DTFIMAPURAC"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row["NRAPURACFAT"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, $row["IDGERANOTA"], Param::PARAM_TYPE_STR));
		$procedure->execute();
	}

	private function procedureApuracRS($row) {
		$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_REFESERV');
		$procedure->addParam(new Param('P_CDFILIAPUR', Param::PARAM_INPUT, $row["CDFILIAL"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDFILIPROC', Param::PARAM_INPUT, $row["CDFILIALLOG"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row["CDCLIENTE"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row["DTINIAPURAC"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row["DTFIMAPURAC"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row["NRAPURACFAT"], Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, "", Param::PARAM_TYPE_STR));
		$procedure->addParam(new Param('P_IDNFCOMPRA', Param::PARAM_INPUT, "S", Param::PARAM_TYPE_STR));
		$procedure->execute();
	}

	public function apurar(DTO\Request $request, DTO\Response $response) {
		try {
			$rows = $request->getDataSet()->getRows();
			$responseDataSet = array ();
			$rowsToSave = $this->getNrApuracFat($rows);
			$this->saveCtrlApurFat($rowsToSave);
			
			foreach ($rows as $key => $row) {
				array_push($responseDataSet, array ());
				array_push($responseDataSet[$key], $row["CDFILIAL"]);
				array_push($responseDataSet[$key], $row["NRAPURACFAT"]);
				array_push($responseDataSet[$key], $row['CDCLIENTE']);
				array_push($responseDataSet[$key], $row['NMRAZSOCCLIE']);
				if ($row["IDGERAPREVIAAPURAC"] === 'S') {
					$row["IDGERANOTA"] = 'N';
				} else {
					$row["IDGERANOTA"] = 'S';					
				}
				$row["GERAPREVIA"] = 'S';
				if ($row["IDCENTRALFATU"] === 'S') {
					$this->executaApuracaoGestaoDiretaCentral($row, $response);
				} else {
					$this->executaApuracaoGestaoDireta($row, $response);
				}
				
				if ($row['UTILIZA_LOCAL'] === 'S') {
					if (empty($this->verificaNotaFiscal($row))) {
						array_push($responseDataSet[$key], "ERRO_SNF");
					} else {
						$cdctrlnumenf = $this->getCdCtrlNumeNF($row);
						$dataSet = $this->verificaIntervalo($row, $cdctrlnumenf);
						
						if (empty($dataSet->getRows()[0]['NRCDFRINI'])) {
							array_push($responseDataSet[$key], "ERRO_NCDRF");
						} else {
							if(!$this->saveLocalEmissao($row, $dataSet, $cdctrlnumenf)) {
								array_push($responseDataSet[$key], "ERRO_CDRF");
							}
						}
					}
				}
			}
			$response->addDataSet(new DataSet("fai70000_apurar", $responseDataSet));
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	public function gerarNF(DTO\Request $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();
			$row["IDGERANOTA"] = 'S';
			$row["GERAPREVIA"] = 'N';
			if ($row["IDCENTRALFATU"] === 'S') {
				$this->executaApuracaoGestaoDiretaCentral($row, $response);
			} else {
				$this->executaApuracaoGestaoDireta($row, $response);
			}
			
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function executaApuracaoGestaoDireta($row, $response) {
		try {
			$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_GESDIRETA');
			$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILIAL"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row["CDCLIENTE"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row["DTINIAPURAC"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row["DTFIMAPURAC"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row["NRAPURACFAT"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, $row["IDGERANOTA"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $row["DTSAIDAPUR"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $row["TIPOOPER"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDSERVFILI', Param::PARAM_INPUT, NULL, Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_GERAPREVIA', Param::PARAM_INPUT, $row["IDGERAPREVIAAPURAC"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDFILIALCENTRAL', Param::PARAM_INPUT, $row['CDFILIAL'],Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACENTRAL', Param::PARAM_INPUT, $row['NRAPURACFAT'],Param::PARAM_TYPE_STR));
			$procedure->execute();
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
    	}
	}

	private function executaApuracaoGestaoDiretaCentral($row, $response) {
		try {
			$procedure = new StoredProcedure($this->entityManager->getConnection(), 'APURAC_GESDIRETA_CENTRAL');
			$procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $row["CDFILIAL"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDCLIENTE', Param::PARAM_INPUT, $row["CDCLIENTE"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAINI', Param::PARAM_INPUT, $row["DTINIAPURAC"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DATAFIM', Param::PARAM_INPUT, $row["DTFIMAPURAC"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_NRAPURACFAT', Param::PARAM_INPUT, $row["NRAPURACFAT"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_IDGERANOTA', Param::PARAM_INPUT, $row["IDGERANOTA"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_DTSAIDAPUR', Param::PARAM_INPUT, $row["DTSAIDAPUR"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_CDTIPOOPER', Param::PARAM_INPUT, $row["TIPOOPER"], Param::PARAM_TYPE_STR));
			$procedure->addParam(new Param('P_GERAPREVIA', Param::PARAM_INPUT, $row["GERAPREVIA"], Param::PARAM_TYPE_STR));
			$procedure->execute();
		} catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
    	}
	}

	public function salvarPreviaApurac(DTO\Request $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$row = $request->getRow();
			$tabela = $row["TABELA"];
			$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
			$tabelaRow = $this->getRowTabela($row, $tabela);
			if (isset($row["CHANGED_CLIENTE"]) && $row["CHANGED_CLIENTE"]) {
				$tabelaRow["CDGRSERVFAT"] = NULL;
			}
			$dataSet = new DataSet($dataSourceName, array($tabelaRow));
			$newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
			if (isset($row["CHANGED_CDTIPOOPER"]) && $row["CHANGED_CDTIPOOPER"]) {
				$this->getgrupoNFRows($row["CDFILIAL"], $row["NRAPURACENTRAL"], $row["CDCLIENTE"], $row["NRGRUPONF"], $row["CDTIPOOPER"], $row["NRSEQUENCIAL"]);
			}
			if (isset($row["CHANGED_NREFETIVOS"]) && $row["CHANGED_NREFETIVOS"]) {
				$rowComplemento = $this->complementaQuantidade($row, $rowsToSave[0], $tabela, $dataSourceName);
				array_push($rowsToSave, $rowComplemento);
			}
			$this->crudService->save($dataSourceName, $rowsToSave);
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function getgrupoNFRows($cdfilial, $nrapuracfat, $cdcliente, $nrgruponf, $cdtipooper, $nrsequencial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			),
			array(
				"columnName" => "CDCLIENTE",
				"operator" => "=",
				"value" => $cdcliente
			),
			array(
				"columnName" => "NRGRUPONF",
				"operator" => "=",
				"value" => $nrgruponf
			),
			array(
				"columnName" => "NRSEQUENCIAL",
				"operator" => "<>",
				"value" => $nrsequencial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai70000_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
		foreach ($rows as $row) {
			$tabela = $row["TABELA"];
			$row["CDTIPOOPER"] = $cdtipooper;
			$rowTabela = $this->getRowTabela($row, $tabela);
			$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
			$dataSet = new DataSet($dataSourceName, array($rowTabela));
			$newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
			$rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
			$this->crudService->save($dataSourceName, $rowsToSave);
		}
	}

	private function getRowTabela($row, $tabela) {
		$changedRow = array (
			"NRSEQUENCIAL"      	   => $row["NRSEQUENCIAL"],
			"CDFILIALAP".$tabela       => $row["CDFILIAL"],
			"CDCLIENTEAP".$tabela      => $row["CDCLIENTE"],
			"CDSERVFILIAP".$tabela     => $row["CDSERVFILI"],
			"VRPRECUNITAP".$tabela     => $row["VRPRECOUNIT"],
			"CDPRODUTOAP".$tabela      => $row["CDPRODUTO"],
			"NREFETIVOSAP".$tabela     => $row["NREFETIVOS"],
			"NRAPURAC".$tabela         => $row["NRAPURAC"],
			"DTAPURACFAT".$tabela      => $row["DTAPURACFAT"],
			"NMSERVICO".$tabela        => $row["NMSERVICO"],
			"CDGRSERVFAT"              => $row["CDGRSERVFAT"],
			"CDTIPOOPER"               => $row["CDTIPOOPER"],
			"VRDESCONTO"               => $row["VRDESCONTO"],
			"DSJUSTIFICATIVA"          => $row["DSJUSTIFICATIVA"],
			"NRGRUPONF"                => $row["NRGRUPONF"],
			"NRPEDIDO"                 => $row["NRPEDIDO"],
			"NRITEMPED"                => $row["NRITEMPED"],
			"CDFILIAGREGADA"           => $row["CDFILIAGREGADA"],
			"NRAPURACENTRAL"           => $row["NRAPURACENTRAL"],
			"__is_new"                 => false
		);
		if ($tabela === "TU") {
			$changedRow["CDTURNOTU"] = $row["CDTURNO"];
		}
		return $changedRow;
	}

	private function complementaQuantidade($row, $newRow, $tabela, $dataSourceName) {
		$newRow["NREFETIVOSAP".$tabela] = intval($row["CHANGED_NREFETIVOS"]) - intval($row["NREFETIVOS"]);
		$newRow["VRDESCONTO"] = 0;
		$newRow["DSJUSTIFICATIVA"] = null;
		$newRow["NRPEDIDO"] = null;
		$newRow["NRITEMPED"] = null;
		$newRow["IDTABELA"] = $tabela;
		$newRow["NRSEQUENCIAL"] = $this->getMaxNrSequencial($row["NRAPURACENTRAL"], $row["CDFILIAL"]);
		$newRow["__is_new"]= true;
		return $newRow;
	}

	private function getMaxNrSequencial($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai70000_trab_apurac_max_nrseq", $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			return $rows[0]["NRSEQUENCIAL"] + 1;
		} else {
			return 1;
		}
	}

	public function agruparItens(DTO\Request $request, DTO\Response $response) {
		try {
			$this->entityManager->getConnection()->beginTransaction();
			$rows = $request->getDataSet()->getRows();
			if (count($rows) === 1) {
				$this->validaItensGrupo($rows[0]["NRAPURACENTRAL"], $rows[0]["CDFILIAL"], $rows[0]["NRGRUPONF"], $rows[0]["NRSEQUENCIAL"]);
			}
			$tabela = $rows[0]["TABELA"];
			$dataSourceName = strtolower("TRAB_APURAC_".$tabela);
			$nrgruponf = $this->getMaxNrGrupoNF($rows[0]["NRAPURACENTRAL"], $rows[0]["CDFILIAL"]);
			foreach($rows as &$row) {
				$row["NRGRUPONF"] = $nrgruponf;
				$newRequest = new DTO\Request\Row($row, "", "", "");
				$this->salvaPreviaApurac($newRequest, $response);
			}
			$dataSet = new DataSet("fai70000_agrupar_itens", array(array("NRGRUPONF" => $nrgruponf)));
			$response->addDataSet($dataSet);
			$this->entityManager->getConnection()->commit();
		} catch (\Exception $e) {
			$this->entityManager->getConnection()->rollback();
			$response->setError(new Error($e->getMessage(), 500));
		}
	}

	private function validaItensGrupo($nrapuracfat, $cdfilial, $nrgruponf, $nrsequencial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			),
			array(
				"columnName" => "NRGRUPONF",
				"operator" => "=",
				"value" => $nrgruponf
			),
			array(
				"columnName" => "NRSEQUENCIAL",
				"operator" => "<>",
				"value" => $nrsequencial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai70000_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
		if (empty($rows)) {
			throw new \Exception("unicoNoGrupo");
		}
	}

	private function getAllTrabApurac($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai70000_trab_apurac", $conditions);
		$rows = $dataSet->getRows();
	}
	
	private function getMaxNrGrupoNF($nrapuracfat, $cdfilial) {
		$conditions = array(
			array(
				"columnName" => "NRAPURACENTRAL",
				"operator" => "=",
				"value" => $nrapuracfat
			),
			array(
				"columnName" => "CDFILIAL",
				"operator" => "=",
				"value" => $cdfilial
			)
		); 
		$dataSet = $this->crudService->getFilteredDataSet("fai70000_trab_apurac_max_nrgruponf", $conditions);
		$rows = $dataSet->getRows();
		if (!empty($rows)) {
			return $rows[0]["NRGRUPONF"] + 1;
		} else {
			return 1;
		}
	}
	
	public function deleteDadosAdic(DTO\Request $request, DTO\Response $response) {
		$row = $request->getRow();
		$cdfilial = $row["CDFILIAL"];
		$nrapuracfat = $row["NRAPURACFAT"];
		$sql = "DELETE FROM DADOSADIC_APURAC WHERE CDFILIAL = :CDFILIAL AND NRAPURACFAT = :NRAPURACFAT AND NRORG = :NRORG";
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRAPURACFAT" => $nrapuracfat,
            "NRORG" => $this->environment->getNrOrg()
        );
        $query = $this->entityManager->getConnection()->prepare($sql);
		$query->execute($params);
	}
}